
import pyawr.mwoffice as mwo  # Import pyawr interface to AWRDE
from itertools import product
import json


###Function turns a keyed dictionary of lists into a iterable list of dictionary elements###
def nest(myDict):
    list = [dict(zip(myDict.keys(), vals)) for vals in product(*myDict.values())]
    return list

###Main Function###
if __name__ == "__main__":

    ###Build a keyed dictionary of lists###
    myDict = {}
    myDict['HPBAND'] = list(range(1, 5, 1))
    myDict['LPBAND'] = list(range(1, 5, 1))
    myDict['HPF'] = list(range(0, 16, 1))
    myDict['LPF'] = list(range(0, 16, 1))
    
    ###Set folder location of 128 s2p files and the target destination###
    hpf_location = 'C:\\...\\hpf\\'
    lpf_location = 'C:\\...\\lpf\\'
    destination = 'C:\\...\\target_s2p\\'
    
    schemName = 'Filter_performance_generation'

    try:
        ###Initialize AWR###
        awr = mwo.CMWOffice('16.0')

        ###Add schematic###
        awrSchem = awr.Project.Schematics(schemName)
        
        ###Grab subcircuits###
        schemSubs = awrSchem.Subcircuits
        hpf = schemSubs.Item(1)
        lpf = schemSubs.Item(2)
        
        for nest in nest(myDict):

            ###Set current state name, current lpf and hpf names###
            ###ADMV8818_T25_HPBAND0_LPBAND0_HPF0_LPF0 is naming convention###
            current_state_name = 'ADMV8818_T25' + '_HPBAND' + str(nest['HPBAND']) + '_LPBAND' + str(
                nest['LPBAND'])  + '_HPF' + str(nest['HPF']) + '_LPF' + str(nest['LPF'])
                
            current_hpf_name =  'ADMV8818_T25' + '_HPBAND' + str(nest['HPBAND']) + '_LPBAND0' + '_HPF' + str(nest['HPF']) + '_LPF0'

            current_lpf_name = 'ADMV8818_T25' + '_HPBAND0' + '_LPBAND' + str(nest['LPBAND']) + '_HPF0' + '_LPF'  + str(nest['LPF'])

            print(current_state_name)
            print(current_hpf_name)
            print(current_lpf_name)

            ###Import HPF and LPF datafiles into MWO###
            awr.Project.DataFiles.Import(Name=current_hpf_name,
                                                   Filename=hpf_location+current_hpf_name+'.s2p',
                                                   Type=0)

            awr.Project.DataFiles.Import(Name=current_lpf_name,
                                                   Filename=lpf_location+current_lpf_name+'.s2p',
                                                   Type=0)

            ###Create LPF and HPF objects and set net name to current .s2p set ex. '"ADMV8818_T25_HPBAND0_LPBAND2_HPF0_LPF15"'###
            hpf.Parameters(2).ValueAsString = json.dumps(current_hpf_name)
            lpf.Parameters(2).ValueAsString = json.dumps(current_lpf_name)

            ###Add output file###
            measurement_name = 'NPORTF(1,1,8,8,50,1,1,' + '\"' + destination + current_state_name + '\"' + ',0)'
            awr.Project.OutputFiles.Add(SourceDoc=schemName, Measurement=measurement_name)

            ###Analyze project###
            awr.Project.Simulator.Analyze()

            ###Cleanup project###
            awr.Project.DataFiles.Remove(current_hpf_name)
            awr.Project.DataFiles.Remove(current_lpf_name)
            awr.Project.OutputFiles.Remove(1)

        print('Device Characterization Reconstruction Complete')

    finally:

        ###Close out any open things that will hang if try fails to properly execute code such as opening a com port.###
        print('Script is finished running')
        pass
        


